
#include "RichString.h"

#include <stdlib.h>
#include <string.h>
#include <curses.h>
#include <assert.h>
#include <sys/param.h>

#include "debug.h"

#define RICHSTRING_MAXLEN 500

/*{

typedef struct RichString_ {
   int len;
   chtype chstr[RICHSTRING_MAXLEN+1];
} RichString;

}*/

/* private property */
WINDOW* workArea = NULL;

RichString RichString_new() {
   RichString this;
   this.len = 0;
   return this;
}

void RichString_delete(RichString this) {
}

RichString RichString_prune(RichString this) {
   this.len = 0;
   return this;
}

void RichString_attrOn(int attrs) {
   wattron(workArea, attrs);
}

void RichString_attrOff(int attrs) {
   wattroff(workArea, attrs);
}

RichString RichString_write(int attrs, char* data) {
   if (!workArea) {
      workArea = newpad(1, 512);
   }
   wattron(workArea, attrs);
   int len = strlen(data);
   mvwaddstr(workArea, 0, 0, data);
   RichString ret = RichString_new();
   ret.len = len;
   mvwinchnstr(workArea, 0, 0, ret.chstr, MIN(RICHSTRING_MAXLEN, len));
   wattroff(workArea, attrs);
   return ret;
}

RichString RichString_append(RichString str, int attrs, char* data) {
   if (!workArea) {
      workArea = newpad(1, 512);
   }
   assert(workArea);
   wattron(workArea, attrs);
   int len = strlen(data);
   mvwaddstr(workArea, 0, 0, data);
   int oldstrlen = str.len;
   str.len += len;
   mvwinchnstr(workArea, 0, 0, str.chstr + oldstrlen, MIN(len, RICHSTRING_MAXLEN - oldstrlen));
   wattroff(workArea, attrs);
   return str;
}

void RichString_setAttr(RichString *str, int attrs) {
   for (int i = 0; i < str->len; i++) {
      char c = str->chstr[i];
      str->chstr[i] = c | attrs;
   }
}

void RichString_applyAttr(RichString *str, int attrs) {
   for (int i = 0; i < str->len; i++) {
      str->chstr[i] |= attrs;
   }
}
